@@ -1,7 +1,7 @@
-__version__ = "0.9.8"
+__version__ = "1.0.0"
 __author__ = "Daniel Cote <dccote@cervo.ulaval.ca>"
 
-__all__ = ["communication", "spectrometers"]
+__all__ = ["communication","motion","oscilloscope","powermeters","spectrometers"]
 
 # We want the modules at the top level
 from .devicemanager import *
@@ -12,6 +12,8 @@ from hardwarelibrary.communication import *
 
 # We want these modules in their namespace
 import hardwarelibrary.spectrometers
+import hardwarelibrary.oscilloscope
 import hardwarelibrary.motion
+import hardwarelibrary.cameras
 
 #import sources #TODO: Not much to see here yet
\ No newline at end of file
